use std::sync::Arc;

use minijinja_autoreload::AutoReloader;

use crate::{handler::internal_server_error, prelude::*};

#[derive(Clone)]
pub struct Context {
    pub db: DatabaseConnection,
    template_loader: Arc<AutoReloader>,
}

impl Context {
    pub fn new(db: DatabaseConnection, template_loader: AutoReloader) -> Context {
        Context {
            db,
            template_loader: Arc::new(template_loader),
        }
    }

    pub fn render<T: Serialize>(&self, path: &str, data: T) -> anyhow::Result<String> {
        // TODO: more graceful handling of the potential errors here; this should not use anyhow
        let env = self.template_loader.acquire_env().unwrap();
        let template = env.get_template(path)?;
        let rendered = template.render(data)?;
        Ok(rendered)
    }

    pub fn render_resp<T: Serialize>(&self, path: &str, data: T) -> Response {
        let rendered = self.render(path, data);
        match rendered {
            Ok(rendered) => Html(rendered).into_response(),
            Err(err) => {
                error!(?err, "error while rendering template");
                internal_server_error()
            }
        }
    }
}
