use async_trait::async_trait;
use axum_login::{AuthUser, AuthnBackend, UserId};

use crate::{
    db::{DbError, UserQuery},
    entity::user,
    password,
    prelude::*,
};

#[derive(Serialize, Deserialize)]
pub struct Credentials {
    pub username: String,
    pub password: String,
}

impl AuthUser for user::Model {
    type Id = i32;

    fn id(&self) -> Self::Id {
        self.id
    }

    fn session_auth_hash(&self) -> &[u8] {
        self.password_hash.as_bytes()
    }
}

#[async_trait]
impl AuthnBackend for Context {
    type User = user::Model;
    type Credentials = Credentials;
    type Error = DbError;

    async fn authenticate(
        &self,
        creds: Self::Credentials,
    ) -> Result<Option<Self::User>, Self::Error> {
        let user = UserQuery(&self.db)
            .by_username(&creds.username)
            .await?
            .filter(|u| password::verify(&u.password_hash, &creds.password));
        Ok(user)
    }

    async fn get_user(&self, user_id: &UserId<Self>) -> Result<Option<Self::User>, Self::Error> {
        UserQuery(&self.db).by_id(*user_id).await
    }
}
